@@ -16,28 +16,14 @@ from urllib.parse import urljoin
 import requests
 
 from distributed import SpecCluster
+from distributed.worker import get_client
 from tornado.ioloop import PeriodicCallback
 
 
 from .backoff import ExpBackoff
+from .plugins import SaturnSetup
 
-try:
-    SATURN_TOKEN = os.environ["SATURN_TOKEN"]
-except KeyError as err:
-    raise RuntimeError(
-        "Required environment variable SATURN_TOKEN not set. "
-        "dask-saturn code should only be run on Saturn Cloud infrastructure."
-    ) from err
 
-try:
-    BASE_URL = os.environ["BASE_URL"]
-except KeyError as err:
-    raise RuntimeError(
-        "Required environment variable BASE_URL not set. "
-        "dask-saturn code should only be run on Saturn Cloud infrastructure."
-    ) from err
-
-HEADERS = {"Authorization": f"token {SATURN_TOKEN}"}
 DEFAULT_WAIT_TIMEOUT_SECONDS = 1200
 
 logfmt = "[%(asctime)s] %(levelname)s - %(name)s | %(message)s"
@@ -51,6 +37,29 @@ handler.setFormatter(logging.Formatter(logfmt, datefmt))
 log.addHandler(handler)
 
 
+def _get_base_url():
+    """Retrieve env var BASE_URL, throwing error if not available"""
+    try:
+        return os.environ["BASE_URL"]
+    except KeyError as err:
+        raise RuntimeError(
+            "Required environment variable BASE_URL not set. "
+            "dask-saturn code should only be run on Saturn Cloud infrastructure."
+        ) from err
+
+
+def _get_headers():
+    """Retrive env var SATURN_TOKEN, throwing error if not available and create headers"""
+    try:
+        SATURN_TOKEN = os.environ["SATURN_TOKEN"]
+    except KeyError as err:
+        raise RuntimeError(
+            "Required environment variable SATURN_TOKEN not set. "
+            "dask-saturn code should only be run on Saturn Cloud infrastructure."
+        ) from err
+    return {"Authorization": f"token {SATURN_TOKEN}"}
+
+
 class SaturnCluster(SpecCluster):
     """
     Create a ``SaturnCluster``, an extension of ``distributed.SpecCluster``
@@ -118,6 +127,13 @@ class SaturnCluster(SpecCluster):
         self.loop = None
         self.periodic_callbacks: Dict[str, PeriodicCallback] = {}
         self.autoclose = autoclose
+        try:
+            self.register_default_plugin()
+        except Exception as e:  # pylint: disable=broad-except
+            log.warning(
+                f"Registering default plugin failed: {e} Hint: you might "
+                "have a different dask-saturn version on your dask cluster."
+            )
 
     @classmethod
     def reset(
@@ -138,7 +154,7 @@ class SaturnCluster(SpecCluster):
         ``help(SaturnCluster)``.
         """
         log.info("Resetting cluster.")
-        url = urljoin(BASE_URL, "api/dask_clusters/reset")
+        url = urljoin(_get_base_url(), "api/dask_clusters/reset")
         cluster_config = {
             "n_workers": n_workers,
             "worker_size": worker_size,
@@ -150,7 +166,7 @@ class SaturnCluster(SpecCluster):
         # only send kwargs that are explicity set by user
         cluster_config = {k: v for k, v in cluster_config.items() if v is not None}
 
-        response = requests.post(url, data=json.dumps(cluster_config), headers=HEADERS)
+        response = requests.post(url, data=json.dumps(cluster_config), headers=_get_headers())
         if not response.ok:
             raise ValueError(response.reason)
         return cls(**cluster_config)
@@ -163,7 +179,7 @@ class SaturnCluster(SpecCluster):
         if self.cluster_url is None:
             return "closed"
         url = urljoin(self.cluster_url, "status")
-        response = requests.get(url, headers=HEADERS)
+        response = requests.get(url, headers=_get_headers())
         if not response.ok:
             return self._get_pod_status()
         return response.json()["status"]
@@ -172,7 +188,7 @@ class SaturnCluster(SpecCluster):
         """
         Status of the KubeCluster pod.
         """
-        response = requests.get(self.cluster_url[:-1], headers=HEADERS)
+        response = requests.get(self.cluster_url[:-1], headers=_get_headers())
         if response.ok:
             return response.json()["status"]
         else:
@@ -208,7 +224,7 @@ class SaturnCluster(SpecCluster):
         ValueError if the scheduler is in a bad state.
         """
         url = urljoin(self.cluster_url, "scheduler_info")
-        response = requests.get(url, headers=HEADERS)
+        response = requests.get(url, headers=_get_headers())
         if not response.ok:
             if self._get_pod_status() in ["error", "closed", "stopped"]:
                 for pc in self.periodic_callbacks.values():
@@ -234,7 +250,7 @@ class SaturnCluster(SpecCluster):
         For documentation on this method's parameters, see
         ``help(SaturnCluster)``.
         """
-        url = urljoin(BASE_URL, "api/dask_clusters")
+        url = urljoin(_get_base_url(), "api/dask_clusters")
         self.cluster_url: Optional[str] = None
 
         cluster_config = {
@@ -251,7 +267,7 @@ class SaturnCluster(SpecCluster):
         expBackoff = ExpBackoff(wait_timeout=scheduler_service_wait_timeout)
         logged_warnings: Dict[str, bool] = {}
         while self.cluster_url is None:
-            response = requests.post(url, data=json.dumps(cluster_config), headers=HEADERS)
+            response = requests.post(url, data=json.dumps(cluster_config), headers=_get_headers())
             if not response.ok:
                 raise ValueError(response.reason)
             data = response.json()
@@ -276,9 +292,16 @@ class SaturnCluster(SpecCluster):
                         "Retry in a few minutes. Check status in Saturn User Interface"
                     )
 
+    def register_default_plugin(self):
+        """Register the default SaturnSetup plugin to all workers."""
+        log.info("Registering default plugins")
+        with get_client(self.scheduler_address) as client:
+            output = client.register_worker_plugin(SaturnSetup())
+            log.info(output)
+
     def _get_info(self) -> Dict[str, Any]:
         url = urljoin(self.cluster_url, "info")
-        response = requests.get(url, headers=HEADERS)
+        response = requests.get(url, headers=_get_headers())
         if not response.ok:
             raise ValueError(response.reason)
         return response.json()
@@ -290,7 +313,7 @@ class SaturnCluster(SpecCluster):
         :param n: number of workers to scale to.
         """
         url = urljoin(self.cluster_url, "scale")
-        response = requests.post(url, json.dumps({"n": n}), headers=HEADERS)
+        response = requests.post(url, json.dumps({"n": n}), headers=_get_headers())
         if not response.ok:
             raise ValueError(response.reason)
 
@@ -298,7 +321,7 @@ class SaturnCluster(SpecCluster):
         """Adapt cluster to have between ``minimum`` and ``maximum`` workers"""
         url = urljoin(self.cluster_url, "adapt")
         response = requests.post(
-            url, json.dumps({"minimum": minimum, "maximum": maximum}), headers=HEADERS
+            url, json.dumps({"minimum": minimum, "maximum": maximum}), headers=_get_headers()
         )
         if not response.ok:
             raise ValueError(response.reason)
@@ -308,7 +331,7 @@ class SaturnCluster(SpecCluster):
         Defines what should be done when closing the cluster.
         """
         url = urljoin(self.cluster_url, "close")
-        response = requests.post(url, headers=HEADERS)
+        response = requests.post(url, headers=_get_headers())
         if not response.ok:
             raise ValueError(response.reason)
         for pc in self.periodic_callbacks.values():
@@ -353,8 +376,8 @@ class SaturnCluster(SpecCluster):
 
 
 def _options() -> Dict[str, Any]:
-    url = urljoin(BASE_URL, "api/dask_clusters/info")
-    response = requests.get(url, headers=HEADERS)
+    url = urljoin(_get_base_url(), "api/dask_clusters/info")
+    response = requests.get(url, headers=_get_headers())
     if not response.ok:
         raise ValueError(response.reason)
     return response.json()["server_options"]
